#!/bin/bash
set -e

function run_program()
{
# This function runs the iterator program

	# Remove left over running file
	rm -f running

	# Write to logfile
	echo "" >> "${myLOGFILE}"
	date >> "${myLOGFILE}"
	echo "Running ${myPROG} with ${@}" >> "${myLOGFILE}"

	# Run program with passed arguments
	"${myPROG}" "${@}" > "stdout-${1}-${2}-${3}-${4}"

	# Remove left over running file
	rm -f running

	# Run gnuplot
	gnuplot "${myGPLT_FILE}"

	# Rename files
	mv output.dat "output-${1}-${2}-${3}-${4}.dat"
	mv output.png "output-${1}-${2}-${3}-${4}.png"

	# Write to logfile
	echo "Output file: output-${1}-${2}-${3}-${4}.dat" >> "${myLOGFILE}"
	echo "Plot of output file: output-${1}-${2}-${3}-${4}.png" >> "${myLOGFILE}"
	echo "Standard output: stdout-${1}-${2}-${3}-${4}" >> "${myLOGFILE}"
}

# Program to run: iterator
myPROG="$(pwd -P)/iterator"

# Location of gnuplot file
myGPLT_FILE="$(pwd -P)/iterator.gplt"

# My current directory
myDIR="$(pwd -P)"

# Location of log file
myLOGFILE="${myDIR}/logfile"

# Temporary directory for me to work in
myTEMP_DIR="$(mktemp -t -d iterator.XXXXXXXXX)"

# Change to temporary directory
cd "${myTEMP_DIR}"

# Read in parameters from standard input
#   and then run program with them
#   and run it again and again until there are no more
while read myNX myNY myN_ITER myEPSILON myJUNK ; do

	# Instead of using read in value for iterations,
	#  use 10, then 100, then 1000.
	zzITER=10
	while [ "${zzITER}" -le "1000" ]  ; do
	
		# Run program
		run_program "${myNX}" "${myNY}" "${zzITER}" "${myEPSILON}"

		zzITER=$(( ${zzITER} * 10 ))
	done

done

# Copy files back to my directory
cp -fpR . "${myDIR}"

# Go back to my directory
cd "${myDIR}"

# Clean up
rm -Rf "${myTEMP_DIR}"
