#!/bin/bash
set -e

function check_args()
{
# This function checks all the arguments it has been given

	# Loop through all the arguments we've been given,
	#   checking each in turn.
	for zzARGS in "${@}" ; do

		# Make sure number of iterations is positive
		if [ "${zzARGS}" -lt "1" ] ; then
			echo "Number of iterations (${zzARGS}) must be positive!" >&2
			exit 1
		fi

		# Make sure number of iterations is not too large
		if [ "${zzARGS}" -gt "10000" ] ; then
			echo "Too many iterations (${zzARGS})!" >&2
			exit 1
		fi

	done
}

function multi_iterate()
{
# Instead of using read in value for iterations,
#  cycle through arguments passed to function.

	for zzITER in "${@}" ; do

		# Run program and report if there were problems
		if ! run_program "${myNX}" "${myNY}" "${zzITER}" "${myEPSILON}" ; then
			echo "Problem with parameter set: ${myNX} ${myNY} ${zzITER} ${myEPSILON}" >&2
		fi

	done
}


function run_program()
{
# This function runs the iterator program

	# Remove left over running file
	rm -f running

	# Write to logfile
	echo "" >> "${myLOGFILE}"
	date >> "${myLOGFILE}"
	echo "Running ${myPROG} with ${@}" >> "${myLOGFILE}"

	# Run program with passed arguments
	set +e
	"${myPROG}" "${@}" > "stdout-${1}-${2}-${3}-${4}"
	myPROG_ERR="${?}"
	set -e

	# Run gnuplot only if the program succeeded
	if [ "${myPROG_ERR}" -eq "0" ] ; then
		set +e
		gnuplot "${myGPLT_FILE}"
		myGPLT_ERR="${?}"
		set -e
	else
		rm -f running
		echo "Failed!  Exit status: ${myPROG_ERR}" >> "${myLOGFILE}"
		return 1
	fi		

	# If gnuplot fails rename output.dat file and quit function
	if [ "${myGPLT_ERR}" -ne "0" ] ; then
		echo "gnuplot failed!  Exit status: ${myGPLT_ERR}" >> "${myLOGFILE}"
		mv output.dat "output-${1}-${2}-${3}-${4}.dat"
		rm -f running
		echo "Output file: output-${1}-${2}-${3}-${4}.dat" >> "${myLOGFILE}"
		return 1
	fi

	# Rename files
	mv output.dat "output-${1}-${2}-${3}-${4}.dat"
	mv output.png "output-${1}-${2}-${3}-${4}.png"

	# Remove left over running file
	rm -f running

	# Write to logfile
	echo "Output file: output-${1}-${2}-${3}-${4}.dat" >> "${myLOGFILE}"
	echo "Plot of output file: output-${1}-${2}-${3}-${4}.png" >> "${myLOGFILE}"
	echo "Standard output: stdout-${1}-${2}-${3}-${4}" >> "${myLOGFILE}"
}

# Program to run: iterator
myPROG="$(pwd -P)/iterator"

# Location of gnuplot file
myGPLT_FILE="$(pwd -P)/iterator.gplt"

# My current directory
myDIR="$(pwd -P)"

# Location of log file
myLOGFILE="${myDIR}/logfile"

# Make sure our command line arguments are okay before continuing
check_args "${@}"

# Temporary directory for me to work in
myTEMP_DIR="$(mktemp -t -d iterator.XXXXXXXXX)"

# Change to temporary directory
cd "${myTEMP_DIR}"

# Read in parameters from standard input
#   and then run program with them
#   and run it again and again until there are no more
while read myNX myNY myN_ITER myEPSILON myJUNK ; do

	if [ -z "${1}" ] ; then
		# If no first command line argument given,
		#  use these defaults.
		echo "Using default number of iterations: 10, 100, 1000"
		multi_iterate "10" "100" "1000"
	else
		# Use the command line arguments
		multi_iterate "${@}"
	fi

done

# Copy files back to my directory
cp -fpR . "${myDIR}"

# Go back to my directory
cd "${myDIR}"

# Clean up
rm -Rf "${myTEMP_DIR}"
